﻿using System;
using System.Diagnostics;
using System.Threading;
using System.Timers;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive.CloudManager.Model;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.Clients.Hive.CloudManager.Views {
  [View("Cloud Resources View")]
  [Content(typeof(IItemList<Subscription>), IsDefaultView = true)]
  public partial class CloudResourcesView : ItemView, IDisposable {
    private System.Timers.Timer timer = new System.Timers.Timer();
    private int updateInterval = 15000;
    private DateTime dueTime;

    public new IItemList<Subscription> Content {
      get { return (IItemList<Subscription>)base.Content; }
      set { base.Content = value; }
    }

    public CloudResourcesView() {
      InitializeComponent();

      CloudManagerClient.Instance.Refreshing += new EventHandler(Instance_Refreshing);
      CloudManagerClient.Instance.Refreshed += new EventHandler(Instance_Refreshed);

      timer.AutoReset = true;
      timer.Interval = 1000;
      timer.Elapsed += PerformUpdate;
      timer.Start();
      dueTime = DateTime.Now.AddMilliseconds(updateInterval);
    }

    public new void Dispose() {
      CloudManagerClient.Instance.Refreshing -= new EventHandler(Instance_Refreshing);
      CloudManagerClient.Instance.Refreshed -= new EventHandler(Instance_Refreshed);
      timer.Dispose();
      Debug.WriteLine("Dispose");
    }

    void Instance_Refreshing(object sender, EventArgs e) {
      Debug.WriteLine("Instance_Refreshing");
    }

    void Instance_Refreshed(object sender, EventArgs e) {
      Debug.WriteLine("Instance_Refreshed");
    }

    private void PerformUpdate(object sender, ElapsedEventArgs e) {
      TimeSpan timeToRefresh = dueTime - DateTime.Now;

      if (timeToRefresh.Seconds >= 0) {
        this.Invoke((MethodInvoker)delegate { lblRefresh.Text = "Done. " + timeToRefresh.Seconds + "s to next refresh."; });
      } else {
        timer.Stop();
        this.Invoke((MethodInvoker)delegate { lblRefresh.Text = "Refreshing data ..."; });

        // statistics
        Debug.WriteLine("perform update");
        Thread.Sleep(1000);

        dueTime = DateTime.Now.AddMilliseconds(updateInterval);
        timer.Start();
      }
    }
  }
}
